#undef TRACEPOINT_PROVIDER
#define TRACEPOINT_PROVIDER sapphire

#if !defined( TP_SAPPHIRE_H ) || defined( TRACEPOINT_HEADER_MULTI_READ )
#define TP_SAPPHIRE_H

#include <lttng/tracepoint.h>
TRACEPOINT_EVENT(
   sapphire,
   regulatory_bin,
   TP_ARGS(
      char const *, session_name_arg,
      char const *, directory_name_arg,
      char const *, payload_arg,
      size_t, payload_len
      ),
   TP_FIELDS(
      ctf_string( session_name, session_name_arg )
      ctf_string( directory_name, directory_name_arg )
      ctf_sequence_hex( unsigned char, payload, payload_arg, size_t, payload_len )
      )
   )

TRACEPOINT_LOGLEVEL( sapphire, regulatory_bin, TRACE_DEBUG_PROCESS )

TRACEPOINT_EVENT(
   sapphire,
   regulatory,
   TP_ARGS(
      char const *, session_name_arg,
      char const *, directory_name_arg,
      char const *, payload_arg,
      size_t, payload_len
      ),
   TP_FIELDS(
      ctf_string( session_name, session_name_arg)
      ctf_string( directory_name, directory_name_arg )
      ctf_sequence_text( unsigned char, binary_payload, payload_arg, size_t, payload_len )
      )
   )

TRACEPOINT_LOGLEVEL( sapphire, regulatory, TRACE_DEBUG_PROCESS )

TRACEPOINT_EVENT(
   sapphire,
   ip_error,
   TP_ARGS(
      int, lineno_arg,
      int, socket_arg,
      int, errno_arg
     ),
   TP_FIELDS(
ctf_integer( int, source_line, lineno_arg )
      ctf_integer( int, socket, socket_arg )
      ctf_integer( int, errno, errno_arg )
      ctf_string( error_text, strerror( errno_arg ) )
      )
   )
TRACEPOINT_LOGLEVEL( sapphire, ip_error, TRACE_ERR )
   
TRACEPOINT_EVENT(
   sapphire,
   ip_error3,
   TP_ARGS(
int, lineno_arg,
      char const *, address_arg,
      int, port_arg,
      int, errno_arg
     ),
   TP_FIELDS(
ctf_integer( int, source_line, lineno_arg )
      ctf_string( address, address_arg )
      ctf_integer( int, port, port_arg )
      ctf_integer( int, errno, errno_arg )
      ctf_string( error_text, strerror( errno_arg ) )
      )
   )

TRACEPOINT_LOGLEVEL( sapphire, ip_error3, TRACE_ERR )

TRACEPOINT_EVENT(
   sapphire,
   msg_mismatch,
   TP_ARGS(
int, lineno_arg,
      int, groupid_arg,
   int, msgtype_arg,
   int, expected_group_arg,
   int, expected_msg_arg
     ),
   TP_FIELDS(
ctf_integer( int, source_line, lineno_arg )
ctf_integer( uint64_t, group_id, groupid_arg )
ctf_integer( uint64_t, msg_type, msgtype_arg )
ctf_integer( uint64_t, expected_group_id, expected_group_arg )
ctf_integer( uint64_t, expected_msg_type, expected_msg_arg )
      )
   )
TRACEPOINT_LOGLEVEL( sapphire, msg_mismatch, TRACE_ERR )

#define SAPPHIRE_ONE_CSTRING_ARG_ARGS    \
   TP_ARGS( \
           int, lineno_arg, \
      char const *, param_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphire,
   one_cstring_param_class,
   SAPPHIRE_ONE_CSTRING_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter, param_arg )
      )
   )

#define SAPPHIRE_1S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      one_cstring_param_class, \
      name, \
      SAPPHIRE_ONE_CSTRING_ARG_ARGS \
      )

#define SAPPHIRE_1S_EVENT( name, level  ) \
   SAPPHIRE_1S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

#define SAPPHIRE_ONE_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg,     \
      uint64_t, param_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphire,
   one_uint_param_class,
   SAPPHIRE_ONE_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, parameter, param_arg )
      )
   )

#define SAPPHIRE_1U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      one_uint_param_class, \
      name, \
      SAPPHIRE_ONE_UINT_ARG_ARGS \
      )

#define SAPPHIRE_1U_EVENT( name, level ) \
   SAPPHIRE_1U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

#define SAPPHIRE_SU_ARG_ARGS    \
   TP_ARGS(                     \
      int, lineno_arg,          \
      char const *, param_arg,  \
      uint64_t, param2_arg      \
   )

TRACEPOINT_EVENT_CLASS(
   sapphire,
   string_uint_param_class,
   SAPPHIRE_SU_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( param1, param_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      )
   )

#define SAPPHIRE_SU_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      string_uint_param_class, \
      name, \
      SAPPHIRE_SU_ARG_ARGS \
      )

#define SAPPHIRE_SU_EVENT( name, level  ) \
   SAPPHIRE_SU_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

#define SAPPHIRE_TWO_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphire,
   two_uint_param_class,
   SAPPHIRE_TWO_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, parameter1, param1_arg )
      ctf_integer( uint64_t, parameter2, param2_arg )
      )
   )

#define SAPPHIRE_TWO_CSTRING_ARG_ARGS    \
   TP_ARGS( \
           int, lineno_arg, \
           char const *, param_arg, \
           char const *, param2_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphire,
   two_cstring_param_class,
   SAPPHIRE_TWO_CSTRING_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter, param_arg )
      ctf_string( parameter2, param2_arg )
      )
   )

#define SAPPHIRE_2S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      two_cstring_param_class, \
      name, \
      SAPPHIRE_TWO_CSTRING_ARG_ARGS \
      )

#define SAPPHIRE_2S_EVENT( name, level  ) \
   SAPPHIRE_2S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

#define SAPPHIRE_2U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      two_uint_param_class, \
      name, \
      SAPPHIRE_TWO_UINT_ARG_ARGS \
      )

#define SAPPHIRE_2U_EVENT( name, level ) \
   SAPPHIRE_2U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

#define SAPPHIRE_THREE_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphire,
   siid_iid_eid_class,
   SAPPHIRE_THREE_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, siid, param1_arg )
      ctf_integer( uint64_t, iid, param2_arg )
      ctf_integer( uint64_t, oid, param3_arg )
      )
   )

#define SAPPHIRE_SIE_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      siid_iid_eid_class, \
      name, \
      SAPPHIRE_THREE_UINT_ARG_ARGS \
      )

#define SAPPHIRE_SIE_EVENT( name, level ) \
   SAPPHIRE_SIE_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

TRACEPOINT_EVENT_CLASS(
   sapphire,
   three_uint_param_class,
   SAPPHIRE_THREE_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, param1, param1_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      ctf_integer( uint64_t, param3, param3_arg )
      )
   )

#define SAPPHIRE_3U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      three_uint_param_class, \
      name, \
      SAPPHIRE_THREE_UINT_ARG_ARGS \
      )

#define SAPPHIRE_3U_EVENT( name, level ) \
   SAPPHIRE_3U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

#define SAPPHIRE_FOUR_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg, \
      uint64_t, param4_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphire,
   four_uint_param_class,
   SAPPHIRE_FOUR_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, param1, param1_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      ctf_integer( uint64_t, param3, param3_arg )
      ctf_integer( uint64_t, param4, param4_arg )
      )
   )

#define SAPPHIRE_4U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphire, \
      four_uint_param_class, \
      name, \
      SAPPHIRE_FOUR_UINT_ARG_ARGS \
      )

#define SAPPHIRE_4U_EVENT( name, level ) \
   SAPPHIRE_4U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphire, name, level )

SAPPHIRE_1S_EVENT( fatal_event,TRACE_EMERG )
SAPPHIRE_1S_EVENT( alert_event,TRACE_ALERT )
SAPPHIRE_1S_EVENT( critcal_event,TRACE_CRIT )
SAPPHIRE_1S_EVENT( error_event,TRACE_ERR )
SAPPHIRE_1S_EVENT( warning_event,TRACE_WARNING )
SAPPHIRE_1S_EVENT( notice_event,TRACE_NOTICE )
SAPPHIRE_1S_EVENT( info_event,TRACE_INFO )
SAPPHIRE_1S_EVENT( debug_system_event,TRACE_DEBUG_SYSTEM )
SAPPHIRE_1S_EVENT( debug_program_event,TRACE_DEBUG_PROGRAM )
SAPPHIRE_1S_EVENT( debug_process_event,TRACE_DEBUG_PROCESS )
SAPPHIRE_1S_EVENT( debug_module_event,TRACE_DEBUG_MODULE )
SAPPHIRE_1S_EVENT( debug_unit_event,TRACE_DEBUG_UNIT )
SAPPHIRE_1S_EVENT( debug_function_event,TRACE_DEBUG_FUNCTION )
SAPPHIRE_1S_EVENT( debug_line_event,TRACE_DEBUG_LINE )
SAPPHIRE_1S_EVENT( debug_event,TRACE_DEBUG )
SAPPHIRE_1S_EVENT( queue_full, TRACE_EMERG )
SAPPHIRE_1S_EVENT( queue_highwatermark, TRACE_ALERT )
SAPPHIRE_1S_EVENT( queue_watermark, TRACE_WARNING )
SAPPHIRE_1S_EVENT( queue_draining, TRACE_INFO )
SAPPHIRE_1S_EVENT( hawk_severe, TRACE_EMERG )
SAPPHIRE_1S_EVENT( hawk_warning, TRACE_WARNING )
SAPPHIRE_1S_EVENT( hawk_info, TRACE_INFO )
SAPPHIRE_1S_EVENT( hawk_error, TRACE_ERR )
SAPPHIRE_1S_EVENT( subscribe_fail, TRACE_ERR )
SAPPHIRE_1S_EVENT( open_fail, TRACE_ERR )
SAPPHIRE_1S_EVENT( instrument_insert_fail, TRACE_ERR )
SAPPHIRE_1S_EVENT( bad_file, TRACE_ERR )
SAPPHIRE_1S_EVENT( bad_session, TRACE_ERR )
SAPPHIRE_1S_EVENT( invalid, TRACE_ERR )
SAPPHIRE_1S_EVENT( listen_fail, TRACE_WARNING )

SAPPHIRE_1U_EVENT( signal_hard_stop, TRACE_EMERG )
SAPPHIRE_1U_EVENT( parent_died, TRACE_EMERG )
SAPPHIRE_1U_EVENT( missed_heartbeat, TRACE_EMERG )
SAPPHIRE_1U_EVENT( received_sigchld, TRACE_EMERG )
SAPPHIRE_1U_EVENT( killing_pid, TRACE_EMERG )
SAPPHIRE_1U_EVENT( out_of_slabs, TRACE_ERR )
SAPPHIRE_1U_EVENT( category_not_found, TRACE_ERR )
SAPPHIRE_1U_EVENT( plugin_failed, TRACE_ERR )
SAPPHIRE_1U_EVENT( orderids_exhausted, TRACE_ERR )
SAPPHIRE_1U_EVENT( timer_remove_failure, TRACE_ERR )
SAPPHIRE_1U_EVENT( queue_full_num, TRACE_ERR )
SAPPHIRE_1U_EVENT( bad_fd, TRACE_ERR )
SAPPHIRE_1U_EVENT( failure, TRACE_ERR )
SAPPHIRE_1U_EVENT( bad_order, TRACE_ERR )
SAPPHIRE_1U_EVENT( bad_iid, TRACE_ERR )
SAPPHIRE_1U_EVENT( deblock_error, TRACE_WARNING )
SAPPHIRE_1U_EVENT( bad_message_type, TRACE_WARNING )
SAPPHIRE_1U_EVENT( bad_write, TRACE_WARNING )
SAPPHIRE_1U_EVENT( affinity_set, TRACE_NOTICE )

SAPPHIRE_SU_EVENT( mq_created, TRACE_INFO )
SAPPHIRE_SU_EVENT( socket_accelerated, TRACE_INFO )
SAPPHIRE_SU_EVENT( bind_failure, TRACE_WARNING )
SAPPHIRE_SU_EVENT( port_bind, TRACE_INFO )
SAPPHIRE_SU_EVENT( listen, TRACE_INFO )
SAPPHIRE_SU_EVENT( connect_fail, TRACE_WARNING )
SAPPHIRE_SU_EVENT( connected, TRACE_INFO )
SAPPHIRE_SU_EVENT( connection_accepted, TRACE_NOTICE )
SAPPHIRE_SU_EVENT( disconnect_from_remote, TRACE_NOTICE )
SAPPHIRE_SU_EVENT( connection_closed, TRACE_NOTICE )

SAPPHIRE_2S_EVENT( bad_sha, TRACE_ERR )
SAPPHIRE_2S_EVENT( bad_overlay, TRACE_ERR )
SAPPHIRE_2S_EVENT( mcast_interface_failed, TRACE_ERR )
SAPPHIRE_2S_EVENT( mcast_join, TRACE_INFO )

SAPPHIRE_2U_EVENT( timer_in_past, TRACE_ERR )
SAPPHIRE_2U_EVENT( node_add_fail, TRACE_ERR )
SAPPHIRE_2U_EVENT( node_find_fail, TRACE_ERR )
SAPPHIRE_2U_EVENT( invalid_orderid_block, TRACE_ERR )
SAPPHIRE_2U_EVENT( sub_failure, TRACE_ERR )
SAPPHIRE_2U_EVENT( unsub_failure, TRACE_ERR )
SAPPHIRE_2U_EVENT( router_client_add_failure, TRACE_ERR )
SAPPHIRE_2U_EVENT( order_not_found, TRACE_ERR )

SAPPHIRE_SIE_EVENT( req_sub_fwd_fail, TRACE_ERR )
SAPPHIRE_SIE_EVENT( req_unsub_fwd_fail, TRACE_ERR )
SAPPHIRE_SIE_EVENT( req_status_feeder_fail, TRACE_ERR )
SAPPHIRE_SIE_EVENT( pod_fill_instrument_not_found, TRACE_ERR )
SAPPHIRE_SIE_EVENT( order_modify_fail, TRACE_ERR )
SAPPHIRE_SIE_EVENT( order_cancel_reject, TRACE_ERR )

SAPPHIRE_3U_EVENT( invalid_pipes, TRACE_ERR )
SAPPHIRE_3U_EVENT( invalid_siid_iid_oid, TRACE_ERR )

SAPPHIRE_4U_EVENT( missed_heartbeat_check, TRACE_WARNING )
SAPPHIRE_4U_EVENT( fill_without_order, TRACE_ERR )
SAPPHIRE_4U_EVENT( failed_to_cancel, TRACE_ERR )

#endif

#undef TRACEPOINT_INCLUDE
#define TRACEPOINT_INCLUDE "apilib/lttng/tp_sapphire.h"

#include <lttng/tracepoint-event.h>
