#undef TRACEPOINT_PROVIDER
#define TRACEPOINT_PROVIDER sapphirelib

#if !defined( TP_SAPPHIRELIB_H ) || defined( TRACEPOINT_HEADER_MULTI_READ )
#define TP_SAPPHIRELIB_H
#include <lttng/tracepoint.h>

#include "strategynodelib/InstrumentInfo.h"
#include "strategynodelib/OrderRecord.h"

#include "apilib/MessageDecoder.h"
#include "apilib/MessageGroupDecoder.h"
#include "RiskControls/DelegatingListener.h"
#include "RiskControls/Decoder.h"
#include "RiskControls/Traits.h"
#include "RiskControls/Messages.h"
#include "RiskControls/Listener.h"
#include "strategynodelib/OrderIdClient.h"

#define SLLOG( event, param ) tracepoint( sapphirelib, event, __LINE__, param )
#define SLLOG2( event, param1, param2 ) tracepoint( sapphirelib, event, __LINE__, param1, param2 )
#define SLLOG3( event, param1, param2, param3 ) tracepoint( sapphirelib, event, __LINE__, param1, param2, param3 )
#define SLLOG4( event, param1, param2, param3, param4 ) tracepoint( sapphirelib, event, __LINE__, param1, param2, param3, param4 )
#define SLLOG5( event, param1, param2, param3, param4, param5 ) \
   tracepoint( sapphirelib, event, __LINE__, param1, param2, param3, param4, param5 )
#define SLLOG6( event, param1, param2, param3, param4, param5, param6 ) \
   tracepoint( sapphirelib, event, __LINE__, param1, param2, param3, param4, param5, param6 )

TRACEPOINT_EVENT(
   sapphirelib,
   book_error,
   TP_ARGS(
      int, lineno_arg,
      int64_t, price_arg,
      uint64_t, qty_arg,
      char const *, text_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( int64_t, price, price_arg )
      ctf_integer( uint64_t, qty, qty_arg )
      ctf_string( text, text_arg )
      )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, book_error, TRACE_ERR )

TRACEPOINT_EVENT(
   sapphirelib,
   book_error1,
   TP_ARGS(
      int, lineno_arg,
      int64_t, price_arg,
      int64_t, price2_arg,
      char const *, text_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( int64_t, price, price_arg )
      ctf_integer( int64_t, price2, price2_arg )
      ctf_string( text, text_arg )
      )
   )
TRACEPOINT_LOGLEVEL( sapphirelib, book_error1, TRACE_ERR )

TRACEPOINT_EVENT(
sapphirelib,
   bad_price_side,
   TP_ARGS(
int, lineno_arg,
   uint64_t, oid_arg,
   int64_t, price_arg,
   uint8_t, side_arg
   ),
   TP_FIELDS(
ctf_integer( int, source_line, lineno_arg )
   ctf_integer( uint64_t, oid, oid_arg )
   ctf_integer( int64_t, price, price_arg )
   ctf_integer( uint8_t, side, side_arg )
   )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, bad_price_side, TRACE_ERR )

TRACEPOINT_EVENT(
   sapphirelib,
   unable_find_factory,
   TP_ARGS(
      int, lineno_arg,
      uint64_t, node_id_arg,
      uint64_t, siid_arg,
      char const *, strategy_type_arg,
      char const *, strategy_version_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, node_id, node_id_arg )
      ctf_integer( uint64_t, siid, siid_arg )
      ctf_string( strategy_type, strategy_type_arg )
      ctf_string( strategy_version, strategy_version_arg )
      )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, unable_find_factory, TRACE_ERR )

TRACEPOINT_EVENT(
   sapphirelib,
   unload_error,
   TP_ARGS(
      int, lineno_arg,
      uint64_t, id_arg,
      char const *, name_arg,
      int64_t, result_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, id, id_arg )
      ctf_string( name, name_arg )
      ctf_integer( uint64_t, result, result_arg )
      )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, unload_error, TRACE_ERR )

TRACEPOINT_EVENT(
   sapphirelib,
   risk_alert,
   TP_ARGS(
      int, lineno_arg,
      char const *, source_arg,
      char const *, text_arg,
      uint64_t, buy_headroom_arg,
      uint64_t, sell_headroom_arg,
      uint64_t, buy_reject_arg,
      uint64_t, sell_reject_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( source, source_arg )
      ctf_string( text, text_arg )
      ctf_integer( uint64_t, buy_headroom, buy_headroom_arg )
      ctf_integer( uint64_t, sell_headroom, sell_headroom_arg )
      ctf_integer( uint64_t, buy_reject_reason, buy_reject_arg )
      ctf_integer( uint64_t, sell_reject_reason, sell_reject_arg )
      )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, risk_alert, TRACE_WARNING )

TRACEPOINT_EVENT(
   sapphirelib,
   risk_alert_msg,
   TP_ARGS(
      int, lineno_arg,
      sr4::RiskControls::RiskAlert const *, msg_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, desk, msg_arg->desk_id() )
      ctf_integer( uint64_t, portfolio, msg_arg->portfolio_id() )
      ctf_integer( uint64_t, strategy, msg_arg->strat_id() )
      ctf_integer( uint64_t, instrument, msg_arg->iid() )
      ctf_integer( uint64_t, exchange, msg_arg->exch_id() )
      ctf_integer( uint64_t, engine, msg_arg->engn_id() )
      ctf_integer_hex( uint64_t, flags, msg_arg->flags() )
      ctf_integer( uint64_t, buy_headroom, msg_arg->buy_headroom() )
      ctf_integer( uint64_t, sell_headroom, msg_arg->sell_headroom() )
      ctf_integer( uint64_t, reason, msg_arg->rej_reason() )
      ctf_string( description, msg_arg->rej_desc().GetCharPtr() )
      )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, risk_alert_msg, TRACE_WARNING )

TRACEPOINT_EVENT(
   sapphirelib,
   wtf_get_next_orderid,
   TP_ARGS(
      int, lineno_arg,
      sr4::strategy::OrderIdClient::order_id_context_t const *, orderid_arg,
      uint64_t, siid_arg,
      uint64_t, block_size_arg,
      uint64_t, order_request_pending_arg
      ),
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, siid, siid_arg )
      ctf_integer( uint64_t, block_size, block_size_arg )
      ctf_integer( uint64_t, order_request_pending, order_request_pending_arg )
      ctf_integer( uint64_t, begin, orderid_arg->begin )
      ctf_integer( uint64_t, end, orderid_arg->end )
      ctf_integer( uint64_t, low, orderid_arg->low )
      ctf_integer( uint64_t, next, orderid_arg->next == nullptr )
      )
   )

TRACEPOINT_LOGLEVEL( sapphirelib, wtf_get_next_orderid, TRACE_WARNING )

#define SAPPHIRELIB_ORDERSTATE_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg,                          \
      uint32_t, node_arg, \
      uint64_t, siid_arg, \
      sr4::strategy::order_record_t const *, record_arg,\
      uint64_t, tag_arg \
      )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   orderstate_param_class,
   SAPPHIRELIB_ORDERSTATE_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint32_t, node_id, node_arg )
      ctf_integer( uint64_t, siid, siid_arg )
      ctf_integer( uint64_t, iid, record_arg->instrument_info->iid )
      ctf_integer( uint64_t, state, (uint8_t)record_arg->state )
      ctf_integer( uint64_t, cloid, record_arg->get_client_order_id() )
      ctf_integer( uint64_t, tid, tag_arg )
      ctf_integer( int64_t, limit, record_arg->pending_limit_price )
      ctf_integer( int64_t, stop, record_arg->pending_stop_price )
      ctf_integer( uint64_t, qty, record_arg->pending_qty )
      )
   )

#define SAPPHIRELIB_ORDERSTATE_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      orderstate_param_class, \
      name, \
      SAPPHIRELIB_ORDERSTATE_ARG_ARGS \
      )

#define SAPPHIRELIB_ORDERSTATE_EVENT( name, level ) \
   SAPPHIRELIB_ORDERSTATE_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_ORDERACTION_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg,                          \
      uint32_t, node_arg, \
      uint64_t, siid_arg, \
      sr4::strategy::order_record_t const *, record_arg,\
      uint64_t, bitfield_arg, \
      uint64_t, cloid_arg \
      )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   orderaction_param_class,
   SAPPHIRELIB_ORDERACTION_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint32_t, node_id, node_arg )
      ctf_integer( uint64_t, siid, siid_arg )
      ctf_integer( uint64_t, iid, record_arg->instrument_info->iid )
      ctf_integer( uint64_t, state, (uint8_t)record_arg->state )
      ctf_integer( uint64_t, cloid, cloid_arg )
      ctf_integer_hex( uint64_t, bitmask, bitfield_arg )
      ctf_integer( int64_t, acked_limit, record_arg->acked_limit_price )
      ctf_integer( int64_t, limit, record_arg->pending_limit_price )
      ctf_integer( int64_t, acked_stop, record_arg->acked_stop_price )
      ctf_integer( int64_t, stop, record_arg->pending_stop_price )
      ctf_integer( uint64_t, acked_qty, record_arg->acked_qty )
      ctf_integer( uint64_t, qty, record_arg->pending_qty )
      )
   )

#define SAPPHIRELIB_ORDERACTION_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      orderaction_param_class, \
      name, \
      SAPPHIRELIB_ORDERACTION_ARG_ARGS \
      )

#define SAPPHIRELIB_ORDERACTION_EVENT( name, level ) \
   SAPPHIRELIB_ORDERACTION_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_ONE_CSTRING_ARG_ARGS    \
   TP_ARGS( \
      int, lineno_arg,        \
      char const *, param_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   one_cstring_param_class,
   SAPPHIRELIB_ONE_CSTRING_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter, param_arg )
      )
   )

#define SAPPHIRELIB_1S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      one_cstring_param_class, \
      name, \
      SAPPHIRELIB_ONE_CSTRING_ARG_ARGS \
      )

#define SAPPHIRELIB_1S_EVENT( name, level  ) \
   SAPPHIRELIB_1S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_ONE_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg,     \
      uint64_t, param_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   one_uint_param_class,
   SAPPHIRELIB_ONE_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, parameter, param_arg )
      )
   )

#define SAPPHIRELIB_1U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      one_uint_param_class, \
      name, \
      SAPPHIRELIB_ONE_UINT_ARG_ARGS \
      )

#define SAPPHIRELIB_1U_EVENT( name, level ) \
   SAPPHIRELIB_1U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_ONE_INT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg,     \
      int64_t, param_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   one_int_param_class,
   SAPPHIRELIB_ONE_INT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( int64_t, parameter, param_arg )
      )
   )

#define SAPPHIRELIB_1I_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      one_int_param_class, \
      name, \
      SAPPHIRELIB_ONE_INT_ARG_ARGS \
      )

#define SAPPHIRELIB_1I_EVENT( name, level ) \
   SAPPHIRELIB_1I_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_SU_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      char const *, param1_arg, \
      uint64_t, param2_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   string_uint_param_class,
   SAPPHIRELIB_SU_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter1, param1_arg )
      ctf_integer( uint64_t, parameter2, param2_arg )
      )
   )

#define SAPPHIRELIB_SU_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      string_uint_param_class, \
      name, \
      SAPPHIRELIB_SU_ARG_ARGS \
      )

#define SAPPHIRELIB_SU_EVENT( name, level ) \
   SAPPHIRELIB_SU_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_TWO_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   two_uint_param_class,
   SAPPHIRELIB_TWO_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, parameter1, param1_arg )
      ctf_integer( uint64_t, parameter2, param2_arg )
      )
   )

#define SAPPHIRELIB_2U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      two_uint_param_class, \
      name, \
      SAPPHIRELIB_TWO_UINT_ARG_ARGS \
      )

#define SAPPHIRELIB_2U_EVENT( name, level ) \
   SAPPHIRELIB_2U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_TWO_INT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      int64_t, param1_arg, \
      int64_t, param2_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   two_int_param_class,
   SAPPHIRELIB_TWO_INT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( int64_t, parameter1, param1_arg )
      ctf_integer( int64_t, parameter2, param2_arg )
      )
   )

#define SAPPHIRELIB_2I_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      two_int_param_class, \
      name, \
      SAPPHIRELIB_TWO_INT_ARG_ARGS \
      )

#define SAPPHIRELIB_2I_EVENT( name, level ) \
   SAPPHIRELIB_2I_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_TWO_CSTRING_ARG_ARGS    \
   TP_ARGS( \
           int, lineno_arg, \
           char const *, param_arg, \
           char const *, param2_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   two_cstring_param_class,
   SAPPHIRELIB_TWO_CSTRING_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter, param_arg )
      ctf_string( parameter2, param2_arg )
      )
   )

#define SAPPHIRELIB_2S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      two_cstring_param_class, \
      name, \
      SAPPHIRELIB_TWO_CSTRING_ARG_ARGS \
      )

#define SAPPHIRELIB_2S_EVENT( name, level  ) \
   SAPPHIRELIB_2S_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_THREE_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   siid_iid_eid_class,
   SAPPHIRELIB_THREE_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, siid, param1_arg )
      ctf_integer( uint64_t, iid, param2_arg )
      ctf_integer( uint64_t, eid, param3_arg )
      )
   )

#define SAPPHIRELIB_SIE_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      siid_iid_eid_class, \
      name, \
      SAPPHIRELIB_THREE_UINT_ARG_ARGS \
      )

#define SAPPHIRELIB_SIE_EVENT( name, level ) \
   SAPPHIRELIB_SIE_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   three_uint_param_class,
   SAPPHIRELIB_THREE_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, siid, param1_arg )
      ctf_integer( uint64_t, iid, param2_arg )
      ctf_integer( uint64_t, eid, param3_arg )
      )
   )

#define SAPPHIRELIB_3U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      three_uint_param_class, \
      name, \
      SAPPHIRELIB_THREE_UINT_ARG_ARGS \
      )

#define SAPPHIRELIB_3U_EVENT( name, level ) \
   SAPPHIRELIB_3U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_UUC_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      char, param3_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   uint_uint_char_class,
   SAPPHIRELIB_UUC_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, param1, param1_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      ctf_integer( uint8_t, param3, param3_arg )
      )
   )

#define SAPPHIRELIB_UUC_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      uint_uint_char_class, \
      name, \
      SAPPHIRELIB_UUC_ARG_ARGS \
      )

#define SAPPHIRELIB_UUC_EVENT( name, level ) \
   SAPPHIRELIB_UUC_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_UUS_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      char const *, param3_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   uint_uint_string_class,
   SAPPHIRELIB_UUS_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, param1, param1_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      ctf_string( param3, param3_arg )
      )
   )

#define SAPPHIRELIB_UUS_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      uint_uint_string_class, \
      name, \
      SAPPHIRELIB_UUS_ARG_ARGS \
      )

#define SAPPHIRELIB_UUS_EVENT( name, level ) \
   SAPPHIRELIB_UUS_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_3US_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg, \
      char const *, param4_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   uint3_string_class,
   SAPPHIRELIB_3US_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, param1, param1_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      ctf_integer( uint64_t, param3, param3_arg )
      ctf_string( param4, param4_arg )
      )
   )

#define SAPPHIRELIB_3US_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      uint3_string_class, \
      name, \
      SAPPHIRELIB_3US_ARG_ARGS \
      )

#define SAPPHIRELIB_3US_EVENT( name, level ) \
   SAPPHIRELIB_3US_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_3IS_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      int64_t, param1_arg, \
      int64_t, param2_arg, \
      int64_t, param3_arg, \
      char const *, param4_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   int3_string_class,
   SAPPHIRELIB_3IS_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( int64_t, param1, param1_arg )
      ctf_integer( int64_t, param2, param2_arg )
      ctf_integer( int64_t, param3, param3_arg )
      ctf_string( param4, param4_arg )
      )
   )

#define SAPPHIRELIB_3IS_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      int3_string_class, \
      name, \
      SAPPHIRELIB_3IS_ARG_ARGS \
      )

#define SAPPHIRELIB_3IS_EVENT( name, level ) \
   SAPPHIRELIB_3IS_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_SUS_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      char const *, param1_arg, \
      uint64_t, param2_arg, \
      char const *, param3_arg \
)

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   string_uint_string_class,
   SAPPHIRELIB_SUS_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( param1, param1_arg )
      ctf_integer( uint64_t, param2, param2_arg )
      ctf_string( param3, param3_arg )
      )
   )

#define SAPPHIRELIB_SUS_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      string_uint_string_class, \
      name, \
      SAPPHIRELIB_SUS_ARG_ARGS \
      )

#define SAPPHIRELIB_SUS_EVENT( name, level ) \
   SAPPHIRELIB_SUS_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_FOUR_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg, \
      uint64_t, param4_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   four_uint_param_class,
   SAPPHIRELIB_FOUR_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, parameter1, param1_arg )
      ctf_integer( uint64_t, parameter2, param2_arg )
      ctf_integer( uint64_t, parameter3, param3_arg )
      ctf_integer( uint64_t, parameter4, param4_arg )
      )
   )

#define SAPPHIRELIB_4U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      four_uint_param_class, \
      name, \
      SAPPHIRELIB_FOUR_UINT_ARG_ARGS \
      )

#define SAPPHIRELIB_4U_EVENT( name, level ) \
   SAPPHIRELIB_4U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_SIX_UINT_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      uint64_t, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg, \
      uint64_t, param4_arg, \
      uint64_t, param5_arg, \
      uint64_t, param6_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   six_uint_param_class,
   SAPPHIRELIB_SIX_UINT_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_integer( uint64_t, parameter1, param1_arg )
      ctf_integer( uint64_t, parameter2, param2_arg )
      ctf_integer( uint64_t, parameter3, param3_arg )
      ctf_integer( uint64_t, parameter4, param4_arg )
      ctf_integer( uint64_t, parameter5, param5_arg )
      ctf_integer( uint64_t, parameter6, param6_arg )
      )
   )

#define SAPPHIRELIB_6U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      six_uint_param_class, \
      name, \
      SAPPHIRELIB_SIX_UINT_ARG_ARGS \
      )

#define SAPPHIRELIB_6U_EVENT( name, level ) \
   SAPPHIRELIB_6U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_3S_G_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      char const *, param1_arg, \
      char const *, param2_arg, \
      char const *, param3_arg, \
      double, param4_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   three_string_double_param_class,
   SAPPHIRELIB_3S_G_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter1, param1_arg )
      ctf_string( parameter2, param2_arg )
      ctf_string( parameter3, param3_arg )
      ctf_float( double, parameter4, param4_arg )
      )
   )

#define SAPPHIRELIB_3S_G_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      three_string_double_param_class, \
      name, \
      SAPPHIRELIB_3S_G_ARG_ARGS \
      )

#define SAPPHIRELIB_3S_G_EVENT( name, level ) \
   SAPPHIRELIB_3S_G_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_3S_2U_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      char const *, param1_arg, \
      char const *, param2_arg, \
      char const *, param3_arg, \
      uint64_t, param4_arg, \
      uint64_t, param5_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   three_string_two_uint_param_class,
   SAPPHIRELIB_3S_2U_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter1, param1_arg )
      ctf_string( parameter2, param2_arg )
      ctf_string( parameter3, param3_arg )
      ctf_integer( uint64_t, parameter4, param4_arg )
      ctf_integer( uint64_t, parameter5, param5_arg )
      )
   )

#define SAPPHIRELIB_3S_2U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      three_string_two_uint_param_class, \
      name, \
      SAPPHIRELIB_3S_2U_ARG_ARGS \
      )

#define SAPPHIRELIB_3S_2U_EVENT( name, level ) \
   SAPPHIRELIB_3S_2U_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

#define SAPPHIRELIB_SUUG_ARG_ARGS \
   TP_ARGS( \
      int, lineno_arg, \
      char const *, param1_arg, \
      uint64_t, param2_arg, \
      uint64_t, param3_arg, \
      double, param4_arg \
   )

TRACEPOINT_EVENT_CLASS(
   sapphirelib,
   suug_param_class,
   SAPPHIRELIB_SUUG_ARG_ARGS,
   TP_FIELDS(
      ctf_integer( int, source_line, lineno_arg )
      ctf_string( parameter1, param1_arg )
      ctf_integer( uint64_t, parameter2, param2_arg )
      ctf_integer( uint64_t, parameter3, param3_arg )
      ctf_float( double, parameter4, param4_arg )
      )
   )

#define SAPPHIRELIB_SUUG_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_EVENT_INSTANCE( \
      sapphirelib, \
      suug_param_class, \
      name, \
      SAPPHIRELIB_SUUG_ARG_ARGS \
      )

#define SAPPHIRELIB_SUUG_EVENT( name, level ) \
   SAPPHIRELIB_SUUG_TRACEPOINT_INSTANCE( name ) \
   TRACEPOINT_LOGLEVEL( sapphirelib, name, level )

SAPPHIRELIB_1U_EVENT( bad_strategy, TRACE_ERR )
SAPPHIRELIB_1U_EVENT( missing_fx_instrument, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( fill_on_bad_order, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( fill_on_bad_instrument, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( order_report_bad_order, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( cancel_bad_order, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( business_reject_bad_order, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( cstoffset_defaulted, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( order_not_claimed, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( no_config_for_strategy, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( timer_added, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( timer_removed, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( bad_instrument_pointer, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( bad_instrument, TRACE_ERR )
SAPPHIRELIB_1U_EVENT( no_delegate_for_request, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( bad_request, TRACE_ERR )
SAPPHIRELIB_1U_EVENT( total_frames, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( total_sequences, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( scope_not_defined_disconnect_delegate, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( scope_not_defined_connect_delegate, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( scope_not_defined_register_source, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( shutdown_request_reason, TRACE_NOTICE )
SAPPHIRELIB_1U_EVENT( shutdown_timer_set, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( no_version_in_response, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( node_id, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( shm_key, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( order_request_complete, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( requesting_n_instruments, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( n_instrument_response, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( startup_time, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( removed_feeder_for_iid, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( adding_channel_monitor, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( removing_channel_monitor, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( unclaimed_positions, TRACE_NOTICE )
SAPPHIRELIB_1U_EVENT( local_port_set, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( heartbeat_timer_set, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( reconnect_timer_set, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( allowed_missed_hb, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( max_reconnect_attempts, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( new_orderstore_file_version, TRACE_NOTICE )
SAPPHIRELIB_1U_EVENT( session_order_request, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( pid_shut_down, TRACE_NOTICE )
SAPPHIRELIB_1U_EVENT( waiting_on_pid, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( signal_caused_shutdown, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( config_request, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( position_reset, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( throttle_set, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( soid_request, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( bad_delegate_on_request, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( removing_unacked_recovery_order, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( request_response, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( request_ended, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( requesting_feeder_channels, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( loading_instruments, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( added_bid_order, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( added_ask_order, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( removed_bid_order, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( removed_ask_order, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( bad_fd, TRACE_ERR )
SAPPHIRELIB_1U_EVENT( order_created_msg_sent, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( exchange_id, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( adding_tradable_sessions, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( load_strategy, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( unload_strategy, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( session_initialize, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( next_outbound_seq, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( next_inbound_seq, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( strategy_risk_alert, TRACE_NOTICE )
SAPPHIRELIB_1U_EVENT( router_tx, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( risk_controls_listener, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( router_commands, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( mass_cancel_detected, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( waiting_on_order_ids, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( registering_strategy_order_sequence, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( order_router_node_shutdown_reason, TRACE_NOTICE )
SAPPHIRELIB_1U_EVENT( strategy_config_request, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( invalid_strategy_in_config_response, TRACE_WARNING )
SAPPHIRELIB_1U_EVENT( subscribe_response, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( unsubscribe_response, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( dispatch, TRACE_INFO )
SAPPHIRELIB_1U_EVENT( order_stacking_complete, TRACE_INFO )

SAPPHIRELIB_1I_EVENT( price_exists, TRACE_WARNING )
SAPPHIRELIB_1I_EVENT( price_outside_book, TRACE_WARNING )
SAPPHIRELIB_1I_EVENT( price_not_found, TRACE_WARNING )

SAPPHIRELIB_1S_EVENT( session_open_fail, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( session_connect_fail, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( not_disconnected, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( logon_send_fail, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( no_exchange_instrument, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( session_not_connected, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( base_plugin_config, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( boot_strap, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( engine_name, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( register_forwarder, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_dispatch_fail, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( queue_subscribed, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( queue_name, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( rx_pipe_name, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( tx_pipe_name, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( product_group_already_registered, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( instrument_name_not_found, TRACE_ERR )
SAPPHIRELIB_1S_EVENT( product_not_found, TRACE_ERR )
SAPPHIRELIB_1S_EVENT( exchange_not_found, TRACE_ERR )
SAPPHIRELIB_1S_EVENT( session_order_request_complete, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( strategy_factory_registered, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( pid_file_cleanup, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( local_if_set, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( shared_services_username, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( orderstore_load_from_file, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( sequence_directory, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( build_version, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( unknown_file, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( default_mic, TRACE_NOTICE )
SAPPHIRELIB_1S_EVENT( feeder_type, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( cached_instrument_response, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( processing_saved_order, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( exch_instrument_changed, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( max_outstanding_requests, TRACE_NOTICE )
SAPPHIRELIB_1S_EVENT( unit_added, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( feeder_channel_added, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( instrument_loaded, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_settings_loaded, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( adding_session_to_routes, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( changing_tag_50, TRACE_NOTICE )
SAPPHIRELIB_1S_EVENT( order_recovery_complete, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( configuration_requested, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_started, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( subscribed, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_reconnect, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( open_session, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( connecting_session, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_disconnected, TRACE_NOTICE )
SAPPHIRELIB_1S_EVENT( disconnect_session, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_handler, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_logon, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_logout, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_heartbeat, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_resend, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_reset, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( session_mass_order_status, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( processed_spread_instrument, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( client_connected, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( client_disconnected, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( symbol, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( bad_config_location, TRACE_WARNING )
SAPPHIRELIB_1S_EVENT( strategy_config_root, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( strategy_configs_in, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( missing_optional_values, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( missing_config, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( saving_config, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( saving_overlay, TRACE_INFO )
SAPPHIRELIB_1S_EVENT( opening_config, TRACE_INFO )

SAPPHIRELIB_SU_EVENT( no_overlay_for_id, TRACE_WARNING )
SAPPHIRELIB_SU_EVENT( unable_apply_overlay_for_id, TRACE_WARNING )
SAPPHIRELIB_SU_EVENT( deblocked, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( custom_total, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( session_order, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( new_global_section, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( new_global_section_result, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( attaching_to_global_section, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( previous_controller_running, TRACE_EMERG )
SAPPHIRELIB_SU_EVENT( instrument_feeder_not_running, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( orderstore_file_loaded, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( pipe_read, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( file_request, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( file_transmit, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( configured_mic, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( instrument_saved, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( instrument_in_ccs2, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( instrument_not_in_ccs2, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( instrument_request, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( new_exchange_instrument, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( timer_based_on_today, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( timer_based_on_yesterday, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( mass_status_request, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( cme_router_defined, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( session_timer_removed, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( logon, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( logoff, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( recovered_order_count, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( requesting_symbol_instrument, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( data_received, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( processed_instrument_contract_size, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( unclaimed_orders_on_strategy, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( timeout_until_orders_unclaimed, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( starting_strategy, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( stopping_strategy, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( strategy_completed, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( local_endpoint_set, TRACE_INFO )
SAPPHIRELIB_SU_EVENT( remote_endpoint_set, TRACE_INFO )

SAPPHIRELIB_2U_EVENT( fix_gap, TRACE_WARNING )
SAPPHIRELIB_2U_EVENT( pending_fix_gap, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( signal_received, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( handle_signal, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( session_order_response, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( mapped_instrument, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( handle_end_request, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( adding_iid_to_channel, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( removing_iid_from_channel, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( fill_and_leaves, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( setting_monitor_interval, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( already_subscribed, TRACE_WARNING )
SAPPHIRELIB_2U_EVENT( removing_feeder, TRACE_NOTICE )
SAPPHIRELIB_2U_EVENT( not_subscribed, TRACE_WARNING )
SAPPHIRELIB_2U_EVENT( setting_signal, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( ignoring_signal, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( mapping_recovery_order, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( mapping_recovered_strategy_order, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( mapping_unacked_recovery_order, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( requesting_instrument, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( find_recovered_order, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( removing_recovered_order, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( bad_exchange_instrument, TRACE_WARNING )
SAPPHIRELIB_2U_EVENT( increment_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( qty_increment_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( denominator_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( sub_denominator_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( depth_trunc_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( implied_trunc_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( display_precision_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( router_scale_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( feeder_scale_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( display_scale_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( settle_scale_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( contract_size_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( maturity_changed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( fd_added, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( fd_removed, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( ccs2_disconnected, TRACE_NOTICE )
SAPPHIRELIB_2U_EVENT( n_of_m, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( wtf_begin_end, TRACE_WARNING )
SAPPHIRELIB_2U_EVENT( start_not_for_me, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( strategy_not_found, TRACE_WARNING )
SAPPHIRELIB_2U_EVENT( strategy_unloaded, TRACE_NOTICE )
SAPPHIRELIB_2U_EVENT( strategy_not_loaded, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( click_trader_message_handler_ctor, TRACE_INFO )
SAPPHIRELIB_2U_EVENT( buffer_length_fragments, TRACE_INFO )

SAPPHIRELIB_2I_EVENT( gross_pos_basis, TRACE_INFO )
SAPPHIRELIB_2I_EVENT( filled_buys_sells, TRACE_INFO )
SAPPHIRELIB_2I_EVENT( working_buys_sells, TRACE_INFO )

SAPPHIRELIB_2S_EVENT( sha_mismatch, TRACE_WARNING )
SAPPHIRELIB_2S_EVENT( reading_file_type, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( null_session_id, TRACE_WARNING )
SAPPHIRELIB_2S_EVENT( ccs2_data, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( group_changed, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( asset_class_changed, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( group_mass_status_request, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( overlay, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( usd_unit_default, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( trading_granted, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( rebinding_risk_control, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( account_override_for_group, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( instrument_on_session, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( session_handler_key, TRACE_INFO )
SAPPHIRELIB_2S_EVENT( evt_status_router2, TRACE_INFO )

SAPPHIRELIB_UUC_EVENT( bad_orderstate, TRACE_ERR )

SAPPHIRELIB_UUS_EVENT( exch_instrument_in_ccs2, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( unknown_order_recovered, TRACE_NOTICE )
SAPPHIRELIB_UUS_EVENT( order_recovered, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( killing_known_order, TRACE_NOTICE )
SAPPHIRELIB_UUS_EVENT( partially_known_order_recovered, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( scheduled_reconnect, TRACE_NOTICE )
SAPPHIRELIB_UUS_EVENT( iid_mass_status_request, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( cancel, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( exch_instrument_not_in_ccs2, TRACE_WARNING )
SAPPHIRELIB_UUS_EVENT( mass_cancel_request, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( order_request, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( download_channel, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( strategy_id_config_request, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( strategy_overlay, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( updating_value_type, TRACE_INFO )
SAPPHIRELIB_UUS_EVENT( market_data_channel_set, TRACE_INFO )

SAPPHIRELIB_SUS_EVENT( unconnected_send, TRACE_WARNING )
SAPPHIRELIB_SUS_EVENT( concord_resubscribe, TRACE_INFO )
SAPPHIRELIB_SUS_EVENT( bad_session_id, TRACE_WARNING )
SAPPHIRELIB_SUS_EVENT( strategybase_constructor, TRACE_INFO )
SAPPHIRELIB_SUS_EVENT( register_strategy, TRACE_INFO )

SAPPHIRELIB_3U_EVENT( fix_decode_error, TRACE_WARNING )
SAPPHIRELIB_3U_EVENT( heartbeat_node_added, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( handle_system_config, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( subscription_request, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( ignoring_subscription_request, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( soid_response, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( disconnect_at_poll, TRACE_NOTICE )
SAPPHIRELIB_3U_EVENT( clip_float_position, TRACE_NOTICE )
SAPPHIRELIB_3U_EVENT( new_file_date, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( file_loaded_date, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( order_router_client, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( pending_modify, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( pending_cancel, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( duplicate_order, TRACE_WARNING )
SAPPHIRELIB_3U_EVENT( prepare_order, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( order_id_exhausted, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( order_id_exhausted_wtf, TRACE_WARNING )
SAPPHIRELIB_3U_EVENT( order_book_ctor, TRACE_INFO )
SAPPHIRELIB_3U_EVENT( order_id_block_request, TRACE_INFO )

SAPPHIRELIB_3US_EVENT( symbol_subscribe, TRACE_INFO )
SAPPHIRELIB_3US_EVENT( feeder_subscribe, TRACE_INFO )
SAPPHIRELIB_3US_EVENT( config_save_request, TRACE_INFO )

SAPPHIRELIB_3IS_EVENT( order_book_side, TRACE_INFO )

SAPPHIRELIB_4U_EVENT( siee, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( position_basis_buys, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( event_subscribe, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( received_instrument_response, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( missing_order_record, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( subscribe_failure, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( cancel_order_sent, TRACE_INFO )
SAPPHIRELIB_4U_EVENT( risk_object_not_returned, TRACE_WARNING )
SAPPHIRELIB_4U_EVENT( evt_status_router1, TRACE_INFO )

SAPPHIRELIB_6U_EVENT( dpsiee, TRACE_INFO )
SAPPHIRELIB_6U_EVENT( event_status, TRACE_INFO )


SAPPHIRELIB_3S_G_EVENT( apply_with_ratio, TRACE_INFO )
SAPPHIRELIB_3S_G_EVENT( rebinding_risk_rule, TRACE_INFO )

SAPPHIRELIB_3S_2U_EVENT( successfully_loaded_strategy, TRACE_INFO )
SAPPHIRELIB_3S_2U_EVENT( strategy_already_loaded, TRACE_INFO )


SAPPHIRELIB_SUUG_EVENT( i_config_stored, TRACE_INFO )

SAPPHIRELIB_ORDERSTATE_EVENT( bad_orderstate_action, TRACE_ERR )
SAPPHIRELIB_ORDERSTATE_EVENT( pending_orderstate_action, TRACE_INFO )

SAPPHIRELIB_ORDERACTION_EVENT( pending_orderaction_modify, TRACE_INFO )

#endif

#undef TRACEPOINT_INCLUDE
#define TRACEPOINT_INCLUDE "tp_sapphirelib.h"

#include <lttng/tracepoint-event.h>
